/*
 * SimpleIDE compatible header file for FlexC
 */

#ifndef SIMPLETOOLS_H_
#define SIMPLETOOLS_H_

#define _IMPL(x) /* NOTHING */

#ifdef __propeller2__
#include <propeller2.h>
#else
#include <propeller.h>
#endif

#include "simplei2c.h"
/*
#define print __builtin_printf
#define printi __builtin_printf
#define putChar(x) __builtin_printf("%c", x)
#define putStr(x) __builtin_printf("%s", x)
#define putDec(x) __builtin_printf("%d", x)
#define putHex(x) __builtin_printf("%x", x)
*/

#define pause(ms) _waitms(ms)

/**
 * @brief Set an I/O pin to output-high
 *
 * @details This function set makes the Propeller
 * P8X32A connect the I/O pin to its positive 3.3 V
 * supply voltage enabling it to source up to
 * 40 mA of current (max 1 W dissipation per chip).
 *
 * @param pin Number of the I/O pin to set high.
 */
void high(int pin);


/**
 * @brief Set an I/O pin to output-low
 *
 * @details This function makes the Propeller
 * P8X32A connect the I/O pin to its ground 0 V
 * supply voltage enabling it to sink up to
 * 40 mA of current (max 1 W dissipation per chip).
 *
 * @param pin Number of the I/O pin to set low.
 */
void low(int pin);


/**
 * @brief Set an I/O pin to input and return 1 if pin
 * detects a high signal, or 0 if it detects low.
 *
 * @details This function makes the Propeller
 * connect the I/O pin to its input buffer
 * so that it can return the binary value of the
 * voltage applied by an external circuit.  
 *
 * @param pin Number of the I/O pin to set to input.
 *
 * @returns 1 to indicate high (above 1.65 V) received 
 * or 0 to indicate low (below 1.65 V) received.
 */
int input(int pin);



/**
 *
 * @name More Individual I/O
 * @{
 */
 
 

/**
 * @brief Toggle the output state of the I/O pin.
 *
 * @details Change I/O pin's output state from low to high
 * or high to low.  This function assumes that some other
 * function has already set the I/O pin to output.
 *
 * @param pin I/O pin number.
 *
 * @returns The new pin state.
 */
unsigned int toggle(int pin);

/**
 * @brief Reverse the direction of an I/O pin.
 *
 * @details If an I/O pin's direction is set to input, this
 * function changes it to output.  If it's set to output,
 * this function changes it to input.
 *
 * @param pin I/O pin number.
 *
 * @returns The new pin direction.
 */
unsigned int reverse(int pin);


/**
 * @brief Check the state of an I/O pin without
 * setting it to input.
 *
 * @details Use this function instead of input if the
 * Propeller needs to maintain an output.  For example,
 * you can use this to monitor another cog's or counter's
 * output signal activity on a pin.  (Note: if the pin
 * is already set to input, it will return the state the
 * external circuit is applying, just like input.)
 *
 * @param pin Number of the I/O pin
 *
 * @returns The pin's state.  If the pin is an output,
 * 1 = 3.3 V and 0 = 0 V.  If the pin is an input,
 * 1 means V > 1.65 V, 0 means it's less.
 */
unsigned int get_state(int pin);

/**
 * @brief Check the direction of the I/O pin.                                        
 *
 * @details This function will tell you the direction of the
 * I/O pin as seen by the cog executing it.  Keep in mind that
 * that your program might make other cogs use the I/O pin as
 * an output, and a cog that treats a pin as an output wins over
 * one that treats it as an input.
 *
 * @param pin I/O pin number
 *
 * @returns I/O pin direction as seen by the cog that runs the
 * function.
 */
unsigned int get_direction(int pin);

/**
 * @brief Get I/O pin output state.
 *
 * @details Keep in mind that this function reports the value in the output
 * register for the cog running the function.  That doesn't tell you if the
 * I/O pin is set to input, or whether another cog is sending a different
 * output state.
 *
 * @param pin I/O pin number.
 *
 * @returns In a register bit for I/O pin, either 1 or 0.
 */
unsigned int get_output(int pin);

/**
 * @brief Set an I/O pin to a given direction.                                        
 *
 * @details This function sets an I/O pin to either output or input.
 *
 * @param pin I/O pin number.
 * @param direction I/O pin direction.
  */
void set_direction(int pin, int direction);

/**
 * @brief Set I/O pin output register bit to either 1 or 0.
 *
 * @details This function focuses on the I/O pin's output register.  If you
 * intend to use it to send high or low signals, consider using high or low
 * functions.  This function can also be used in conjunction with set_direction
 * to send high or low signals.
 *
 * @param pin I/O pin to set high or low.
 * @param state 1 for high, 0 for low (when pin is actually set to output,
 * which can be done with setDirection.
 */
void set_output(int pin, int state);



/**
 * @}
 *
 * @name Group I/O
 * @{
 */



/**
 * @brief Get states of a contiguous group of I/O pins
 *
 * @details This works the same as getState, but for a group of pins.  It
 * tells you the actual state of each pin, regardless of whether it's a 
 * voltage applied to input or transmitted by an output.
 *
 * @param endPin The highest numbered pin.
 * @param startPin The lowest numbered pin.
 *
 * @returns States value containing the binary bit pattern.  The value for
 * startPin should be in bit-0, next in bit-1, etc.
 */
unsigned int get_states(int endPin, int startPin);

/**
 * @brief Get directions for a contiguous group of I/O pins.
 *
 * @details Get direction register states from a contiguous group of bits 
 * in the cog's output register.
 *
 * @param endPin The highest numbered pin.
 * @param startPin The lowest numbered pin.
 *
 * @returns States value containing a binary bit pattern.  The value for
 * startPin should be in bit-0, next in bit-1, etc.
 */
unsigned int get_directions(int endPin, int startPin);

/**
 * @brief Get output settings for a contiguous group of I/O pins.
 *
 * @details Get output register settings for a contiguous group of bits 
 * in the cog's output register.
 *
 * @param endPin The highest numbered pin.
 * @param startPin The lowest numbered pin.
 *
 * @returns Pattern value containing a binary bit pattern.  The value 
 * for startPin should be in bit-0, next in bit-1, etc.
 */
unsigned int get_outputs(int endPin, int startPin);

/**
 * @brief Set directions for a contiguous group of I/O pins.
 *
 * @details Set directions values in a contiguous group of bits in the 
 * cog's output register.
 *
 * @param endPin The highest numbered pin.
 * @param startPin The lowest numbered pin.
 * @param pattern Value containing the binary bit pattern.  The value for
 * startPin should be in bit-0, next in bit-1, etc.
 */
void set_directions(int endPin, int startPin, unsigned int pattern);

/**
 * @brief Set output states for a contiguous group of I/O pins.
 *
 * @details Set output states of a contiguous group of bits in the cog's
 * output register.
 *
 * @param endPin The highest numbered pin.
 * @param startPin The lowest numbered pin.
 * @param pattern Value containing the binary bit pattern.  The value for
 * startPin should be in bit-0, next in bit-1, etc.
 */
void set_outputs(int endPin, int startPin, unsigned int pattern);


// Constants for shift_in & shift_out
#define   MSBPRE     0
#define   LSBPRE     1
#define   MSBPOST    2
#define   LSBPOST    3
#define   LSBFIRST   0
#define   MSBFIRST   1

int shift_in(int pinDat, int pinClk, int mode, int bits) _IMPL("libsimpletools/shiftIn.c");
void shift_out(int pinDat, int pinClk, int mode, int bits, int value) _IMPL("libsimpletools/shiftOut.c");

// Variable set by i2c_newbus.
extern unsigned int st_buscnt;

i2c *i2c_newbus(int sclPin, int sdaPin, int sclDrive) _IMPL("libsimpletools/i2c_init.c");
int i2c_out(i2c *busID, int i2cAddr, int memAddr, int memAddrCount, const unsigned char *data, int dataCount) _IMPL("libsimpletools/i2c_out.c");
int i2c_in(i2c *busID, int i2cAddr, int memAddr, int memAddrCount, unsigned char *data, int dataCount) _IMPL("libsimpletools/i2c_in.c");
int i2c_busy(i2c *busID, int i2cAddr) _IMPL("libsimpletools/i2c_busy.c");

// Variables set/used by ee_config
extern i2c *st_eeprom;
extern int st_eeInitFlag;

#define EEPROM_ADDR	 0x50

#define ee_put_byte ee_putByte
#define ee_get_byte ee_getByte
#define ee_put_int ee_putInt
#define ee_get_int ee_getInt
#define ee_put_str ee_putStr
#define ee_get_str ee_getStr
#define ee_put_float32 ee_putFloat32

void ee_config(int sclPin, int sdaPin, int sclDrive) _IMPL("libsimpletools/eeprom_initSclDrive.c");
void ee_init() _IMPL("libsimpletools/eeprom_init.c");
void ee_putByte(unsigned char value, int addr) _IMPL("libsimpletools/eeprom_putByte.c");
char ee_getByte(int addr) _IMPL("libsimpletools/eeprom_getByte.c");
void ee_putInt(int value, int addr) _IMPL("libsimpletools/eeprom_putInt.c");
int ee_getInt(int addr) _IMPL("libsimpletools/eeprom_getInt.c");
void ee_putStr(unsigned char *s, int n, int addr) _IMPL("libsimpletools/eeprom_putStr.c");
unsigned char* ee_getStr(unsigned char* s, int n, int addr) _IMPL("libsimpletools/eeprom_getStr.c");
void ee_putFloat32(float value, int addr) _IMPL("libsimpletools/eeprom_putFloat.c");
float ee_getFloat32(int addr) _IMPL("libsimpletools/eeprom_getFloat.c");

float constrainFloat(float value, float min, float max) _IMPL("libsimpletools/constrainFloat.c"); 
int constrainInt(int value, int min, int max) _IMPL("libsimpletools/constrainInt.c");

void endianSwap(void *resultAddr, void *varAddr, int byteCount) _IMPL("libsimpletools/endianSwap.c");

float mapFloat(float value, float fromMin, float fromMax, float toMin, float toMax) _IMPL("libsimpletools/mapFloat.c");
int mapInt(int value, int fromMin, int fromMax, int toMin, int toMax) _IMPL("libsimpletools/mapInt.c");

int random(int limitLow, int limitHigh) _IMPL("libsimpletools/random.c");

#endif
